#include "plumpconfig.h"

int plump_breakpointIsWatching(plump_t *p, plumpaddr_t addr)
{
	int ret = PLUMP_ERROR_FAILURE;
	unsigned long x;

	for (x = 0; 
			ret != PLUMP_SUCCESS && x < p->numUserBreakpoints;
			x++)
	{
		if (p->userBreakpoints[x].addr == addr && p->userBreakpoints[x].enabled)
			ret = PLUMP_SUCCESS;
	}

	return ret;
}

int plump_breakpointAdd(plump_t *p, plumpaddr_t addr)
{
	int ret = PLUMP_ERROR_NOTSUPPORTED;

	if (!(p->userBreakpoints = (plumpbp_t *)realloc(p->userBreakpoints, sizeof(plumpbp_t) * ++p->numUserBreakpoints)))
		return PLUMP_ERROR_NOMEMORY;

	p->userBreakpoints[p->numUserBreakpoints - 1].addr    = addr;
	p->userBreakpoints[p->numUserBreakpoints - 1].enabled = 1;

	ret = PLUMP_SUCCESS;

	return ret;
}

int plump_breakpointEnable(plump_t *p, plumpaddr_t addr)
{
	int ret = PLUMP_ERROR_NOTSUPPORTED;
	unsigned long x;

	for (x = 0; 
			x < p->numUserBreakpoints;
			x++)
	{
		if (p->userBreakpoints[x].addr == addr)
			p->userBreakpoints[x].enabled = 1;
	}

	return ret;
}

int plump_breakpointDisable(plump_t *p, plumpaddr_t addr)
{
	int ret = PLUMP_ERROR_NOTSUPPORTED;
	unsigned long x;

	for (x = 0; 
			x < p->numUserBreakpoints;
			x++)
	{
		if (p->userBreakpoints[x].addr == addr)
			p->userBreakpoints[x].enabled = 0;
	}

	return ret;
}

int plump_breakpointRemove(plump_t *p, plumpaddr_t addr)
{
	plumpbp_t *shrinkAt = NULL, *next = NULL;
	int ret = PLUMP_SUCCESS, pre = 0, post = 0;
	unsigned long x;

	for (x = 0; 
			!shrinkAt && x < p->numUserBreakpoints;
			x++)
	{
		if (p->userBreakpoints[x].addr == addr)
			shrinkAt = p->userBreakpoints + x;
	}

	if (!shrinkAt)
		return PLUMP_ERROR_FAILURE;

	if (--p->numUserBreakpoints)
	{
		if (!(next = (plumpbp_t *)malloc(sizeof(plumpbp_t) * --p->numUserBreakpoints)))
			return PLUMP_ERROR_NOMEMORY;

		// Determine the number of before/after elements
		pre  = shrinkAt - p->userBreakpoints;
		post = p->numUserBreakpoints - pre;
	
		// Re-organize the array
		if (pre)
			memcpy(next, p->userBreakpoints, pre * sizeof(plumpbp_t));
		if (post)
			memcpy(next + (pre * sizeof(plumpbp_t)), shrinkAt + 1, post * sizeof(plumpbp_t));
	
		free(p->userBreakpoints);
	
		// Update the breakpoint array
		p->userBreakpoints = next;
	}
	else
		p->userBreakpoints = NULL;

	return ret;
}
