#include "plumpconfig.h"

int plump_registersSet(plump_t *p, plumpreg_t *registers, unsigned long numRegisters)
{
	int ret = PLUMP_ERROR_NOTSUPPORTED;

	if (p->vftable.registersSet)
		ret = p->vftable.registersSet(p, registers, numRegisters);

	return ret;
}

int plump_registersGet(plump_t *p, plumpreg_t *registers, unsigned long numRegisters)
{
	int ret = PLUMP_ERROR_NOTSUPPORTED;

	if (p->vftable.registersGet)
		ret = p->vftable.registersGet(p, registers, numRegisters);

	return ret;
}

int plump_registersGetStackPointer(plump_t *p, plumpreg_t *sp)
{
	sp[0].name = REGISTER_TYPE_STACK_POINTER;

	return plump_registersGet(p, sp, 1);
}

int plump_registersSetStackPointer(plump_t *p, plumpreg_t *sp)
{
	sp[0].name = REGISTER_TYPE_STACK_POINTER;

	return plump_registersSet(p, sp, 1);
}

int plump_registersGetFramePointer(plump_t *p, plumpreg_t *fp)
{
	fp[0].name = REGISTER_TYPE_FRAME_POINTER;

	return plump_registersGet(p, fp, 1);
}

int plump_registersSetFramePointer(plump_t *p, plumpreg_t *fp)
{
	fp[0].name = REGISTER_TYPE_FRAME_POINTER;

	return plump_registersSet(p, fp, 1);
}

int plump_registersGetProgramCounter(plump_t *p, plumpreg_t *pc)
{
	pc[0].name = REGISTER_TYPE_PROGRAM_COUNTER;

	return plump_registersGet(p, pc, 1);
}

int plump_registersSetProgramCounter(plump_t *p, plumpreg_t *pc)
{
	pc[0].name = REGISTER_TYPE_PROGRAM_COUNTER;

	return plump_registersSet(p, pc, 1);
}

int plump_registersTranslate(unsigned long type, void *reg, plumpreg_t *want, unsigned long numWanted, unsigned long read)
{
	unsigned long x = 0;
	void *offset;

	for (x = 0;
			x < numWanted;
			x++)
	{
		offset = NULL;

		switch (type)
		{
#if defined(OS_LINUX)
			case REGISTER_STRUCT_TYPE_USER_REGS_STRUCT:
				if ((REGISTER_TYPE(want[x].name) == REGISTER_TYPE_FRAME_POINTER) || 
							(REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EBP)))
					offset = &((struct user_regs_struct *)reg)->ebp;
				else if ((REGISTER_TYPE(want[x].name) == REGISTER_TYPE_STACK_POINTER) || 
							(REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ESP)))
					offset = &((struct user_regs_struct *)reg)->esp;
				else if ((REGISTER_TYPE(want[x].name) == REGISTER_TYPE_PROGRAM_COUNTER) || 
							(REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EIP)))
					offset = &((struct user_regs_struct *)reg)->eip;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EAX)) 
					offset = &((struct user_regs_struct *)reg)->eax;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EBX)) 
					offset = &((struct user_regs_struct *)reg)->ebx;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ECX)) 
					offset = &((struct user_regs_struct *)reg)->ecx;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EDX)) 
					offset = &((struct user_regs_struct *)reg)->edx;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EDI)) 
					offset = &((struct user_regs_struct *)reg)->edi;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ESI)) 
					offset = &((struct user_regs_struct *)reg)->esi;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_SS)) 
					offset = &((struct user_regs_struct *)reg)->ss;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_CS)) 
					offset = &((struct user_regs_struct *)reg)->cs;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_DS)) 
					offset = &((struct user_regs_struct *)reg)->ds;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ES)) 
					offset = &((struct user_regs_struct *)reg)->es;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_FS)) 
					offset = &((struct user_regs_struct *)reg)->fs;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_GS)) 
					offset = &((struct user_regs_struct *)reg)->gs;
				break;
#elif defined(OS_FREEBSD)
			case REGISTER_STRUCT_TYPE_REG:
				if ((REGISTER_TYPE(want[x].name) == REGISTER_TYPE_FRAME_POINTER) || 
							(REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EBP)))
					offset = &((struct reg *)reg)->r_ebp;
				else if ((REGISTER_TYPE(want[x].name) == REGISTER_TYPE_STACK_POINTER) || 
							(REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ESP)))
					offset = &((struct reg *)reg)->r_esp;
				else if ((REGISTER_TYPE(want[x].name) == REGISTER_TYPE_PROGRAM_COUNTER) || 
							(REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EIP)))
					offset = &((struct reg *)reg)->r_eip;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EAX)) 
					offset = &((struct reg *)reg)->r_eax;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EBX)) 
					offset = &((struct reg *)reg)->r_ebx;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ECX)) 
					offset = &((struct reg *)reg)->r_ecx;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EDX)) 
					offset = &((struct reg *)reg)->r_edx;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_EDI)) 
					offset = &((struct reg *)reg)->r_edi;
				else if (REGISTER_NAME(want[x].name) == REGISTER_NAME(REGISTER_IA32_ESI)) 
					offset = &((struct reg *)reg)->r_esi;
				break;

#endif
			default:
				break;
		}

		if (offset)
			(read) ? (want[x].val.b32 = *(unsigned long *)offset) : (*(unsigned long *)offset = want[x].val.b32);

	}

	return PLUMP_SUCCESS;
}
